#ifndef ENERGY_ESTIMATOR_H
#define ENERGY_ESTIMATOR_H

#define VERSION_STRING "energy_estimator-1.0"
#include <inttypes.h>
#include <stdio.h>
#include <sys/time.h>

// 06_2AH : Intel Core Sandy Bridge
// 06_2DH : Intel Xeon Sandy Bridge 
   
/* MSRs that are supported on both 062A and 062D */
#define MSR_RAPL_POWER_UNIT			0x606

#define MSR_PKG_RAPL_POWER_LIMIT	0x610
#define MSR_PKG_ENERGY_STATUS		0x611
#define MSR_PKG_PERF_STATUS			0x613
#define MSR_PKG_POWER_INFO			0x614

#define MSR_PP0_POWER_LIMIT			0x638
#define MSR_PP0_ENERGY_STATUS		0x639
#define MSR_PP0_POLICY				0x63A
#define MSR_PP0_PERF_STATUS			0x63B

/* MSRs that are supported on 062A */
#define MSR_PP1_POWER_LIMIT			0x640
#define MSR_PP1_ENERGY_STATUS		0x641
#define MSR_PP1_POLICY				0x642

/* MSRs that are supported on 062D */
#define MSR_DRAM_POWER_LIMIT		0x618
#define MSR_DRAM_ENERGY_STATUS		0x619
#define MSR_DRAM_PERF_STATUS		0x61B
#define MSR_DRAM_POWER_INFO			0x61C

#define WRAP_AROUND_VALUE			0xffffffff

#ifndef BIPRO

// JMCG SINGLE PROCESSOR 

typedef struct 
{
	int fd;
	FILE *fp;
	double core_energy;
	double package_energy;
	double pp1_energy;
	uint64_t reg_pkg_start;
	uint64_t reg_pkg_end;
	uint64_t reg_pp0_start;
	uint64_t reg_pp0_end;	
	uint64_t reg_pp1_start;
	uint64_t reg_pp1_end;	
	struct timeval tv_begin;
	struct timeval tv_end;
	struct timeval tv_diff;
} energy_counter;

#else

// JMCG TWO PROCESSORS ON THE SAME BOARD

typedef struct 
{
	int fd[2];
	FILE *fp;
	double core_energy;
	double package_energy;
	double pp1_energy;
	uint64_t reg_pkg_start[2];
	uint64_t reg_pkg_end[2];
	uint64_t reg_pp0_start[2];
	uint64_t reg_pp0_end[2];	
	uint64_t reg_pp1_start[2];
	uint64_t reg_pp1_end[2];	
	struct timeval tv_begin;
	struct timeval tv_end;
	struct timeval tv_diff;
} energy_counter;


#endif

int initialize(energy_counter *eng_ctr, FILE* foutput);
int start_reading(energy_counter *eng_ctr);
int stop_reading(energy_counter *eng_ctr);
void estimate_energy(energy_counter *eng_ctr);
void finalize(energy_counter *eng_ctr);

#endif
