/**
 * The header file of the plotter.cpp program 
 * for plotting linegraph, histogram and whiskerplot 
 * with gnuplot.
 * Prepared by Abdullah Al Hasib
 */

#ifndef _PLOTTER_H_
#define _PLOTTER_H_

#ifdef _WIN32
	#define GNUPLOT_CMD "E:/GNU_soft/bin/pgnuplot -persist"
#else
	#define GNUPLOT_CMD "/usr/bin/gnuplot -persist"
#endif

#include <cstdlib>
#include <iostream>

using namespace std ;

class Plotter
{
	private:
		int size; // size of the buffer
		static int count; // counts the number of plotter object instances
		static FILE *gp; // gnuplot interface pipe
		char fileName[80]; // the dump-file name for plotting
		double *cpu_eng, *pkg_eng, *time, *cpu_edp, *pkg_edp; // buffer to contain energy values
		double mean_cpu, median_cpu, std_dev_cpu, first_quart_cpu, third_quart_cpu, outliers_cpu;
		double mean_pkg, median_pkg, std_dev_pkg, first_quart_pkg, third_quart_pkg, outliers_pkg;		
		
	public:
		Plotter(); // empty constructor
		Plotter(int size); // plotter with a specified buffer size
		~Plotter(); //destructor
		static void initEnv(); // starts the plotting environment
		static void closeEnv(); // closes the plotting environment
		void load(char *path); // load the buffers from another source
        void lineplot(char title[], char lbl_x[], char lbl_y[], char leg_a[], char leg_b[], int xtics, int col_a, int col_b); // plot the outputs using lineplot
        void histogram(int minx, int maxx, int miny, int maxy); // plot the outputs using histogram
        void candlestick(); // plot the outputs using candlestick
		void computeStats(); // calculate the mean, median, quartiles & std. deviation
		void save(char *path); // save the statistic result to a file
        void save(char *path, char *format); // save the plot to an image file
};

#endif
