#!/bin/bash

parameters=(
"test csx"
"path ./csx/cg"
"exec cg"
"testset exp_setup"
"reps 1"
"yokogawa_log 0"
"sampling_portion 0.01"
"symmetry 0"
"cg_iterations 1024"
"cg_reps 1"
)

# statistical preprocessing
total_samples=64

cpu_freqs=(
"1200"
"1400"
"1600"
"1800"
"2000"
"2200"
"2400"
"2600"
)

matrices=(
"./csx/matrices/2cubes_sphere/2cubes_sphere.mtx.sorted"
"./csx/matrices/af_5_k101/af_5_k101.mtx.sorted"
"./csx/matrices/af_shell3/af_shell3.mtx.sorted"
"./csx/matrices/bone010/bone010.mtx.sorted"
"./csx/matrices/boneS01/boneS01.mtx.sorted"
"./csx/matrices/boneS10/boneS10.mtx.sorted"
"./csx/matrices/gyro/gyro.mtx.sorted"
"./csx/matrices/LF10000/LF10000.mtx.sorted"
"./csx/matrices/nasa2146/nasa2146.mtx.sorted"
"./csx/matrices/nasa2910/nasa2910.mtx.sorted"
"./csx/matrices/nasa4704/nasa4704.mtx.sorted"
"./csx/matrices/nasasrb/nasasrb.mtx.sorted"
"./csx/matrices/offshore/offshore.mtx.sorted"
"./csx/matrices/olafu/olafu.mtx.sorted"
"./csx/matrices/parabolic_fem/parabolic_fem.mtx.sorted"
"./csx/matrices/Pres_Poisson/Pres_Poisson.mtx.sorted"
"./csx/matrices/raefsky4/raefsky4.mtx.sorted"
"./csx/matrices/smt/smt.mtx.sorted"
"./csx/matrices/sts4098/sts4098.mtx.sorted"
"./csx/matrices/thermal1/thermal1.mtx.sorted"
"./csx/matrices/thermal2/thermal2.mtx.sorted"
)

matr_no_stat=(
"./csx/matrices/LF10000/LF10000.mtx.sorted"
"./csx/matrices/nasa2146/nasa2146.mtx.sorted"
"./csx/matrices/nasa2910/nasa2910.mtx.sorted"
"./csx/matrices/nasa4704/nasa4704.mtx.sorted"
"./csx/matrices/olafu/olafu.mtx.sorted"
"./csx/matrices/Pres_Poisson/Pres_Poisson.mtx.sorted"
"./csx/matrices/raefsky4/raefsky4.mtx.sorted"
"./csx/matrices/sts4098/sts4098.mtx.sorted"
)

xform_confs=(
"0"
"1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22"
)

compress=(
"0"
"1"
)

mt_confs=(
"0"
"0,1"
"0,8"
"0,16"
"0,1,2,3"
"0,1,8,9"
"0,1,16,17"
"0,1,2,3,4,5,6,7"
"0,1,2,3,8,9,10,11"
"0,1,2,3,16,17,18,19"
"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15"
"0,1,2,3,4,5,6,7,16,17,18,19,20,21,22,23"
"0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31"
)

mt_threads=(
1
2
2
2
4
4
4
8
8
8
16
16
32
)

for id in ${!parameters[*]} ; do
	echo ${parameters[$id]}
done

for freq in ${cpu_freqs[*]} ; do
	echo "cpufreq ${freq}"
	for matrix in ${matrices[*]} ; do
		echo "matrix ${matrix}"
		for mtid in ${!mt_confs[*]} ; do
			echo "mt_conf ${mt_confs[$mtid]}"
			for xid in ${!xform_confs[*]} ; do
				echo "compress ${compress[$xid]}"
				echo "xform_conf ${xform_confs[$xid]}"
				if [ ${compress[$xid]} = "1" ] ; then
					echo "enable_stat_preprocessing 0"
					echo "run"
					# Some matrices cannot do stat preprocessing
					if ! [[ ${matr_no_stat[*]} =~ $matrix ]]; then
						echo "enable_stat_preprocessing 1"
						echo "samples $(($total_samples/${mt_threads[$mtid]}))"
						echo "run"
					fi
				else
					echo "enable_stat_preprocessing 0"
					echo "run"
				fi
			done
		done
	done
done
