import matplotlib.pyplot as plt
import sys
import os
import numpy as np
import sqlite3
import re
import glob
from math import sqrt
import datetime

from scipy.stats import pearsonr

from plot_configurations import *

results = {}
CARD_INPUT = None
VILJE_INPUT = None
OUTPUT_DIR = None

bar_colors = [ 'b', 'r', 'g', 'c', 'm', 'y', 'k']

# Return conn and cursor
def connect_to_db(dbname):
    conn = sqlite3.connect(dbname)
    return (conn,conn.cursor())

# Create dictionary structure
def fill_dbresults(db_data, server_name):
    # row[0] = matrix, row[1] = mt_conf, row[2] = xform_conf, row[3] = enable_stat_preprocessing,
    # row[4] = cg_execution_time_mean, row[5] = cg_execution_time_stddev,
    # row[6] = papi_l2_refs_mean, row[7] = papi_l2_refs_stddev, row[8] = papi_l2_miss_mean, row[9] = papi_l2_miss_stddev,
    # row[10] = papi_l3_refs_mean, row[11] = papi_l3_refs_stddev, row[12] = papi_l3_miss_mean, row[13] = papi_l3_miss_stddev,
    for row in db_data:
        # remove path
        matrix = re.sub('\.\/csx\/matrices\/\S+\/', '', row[0])
        # remove .mtx.sorted
        matrix = matrix[:-11]
        if server_name not in results.keys():
            results[server_name] = {}
        if matrix not in results[server_name].keys():
            results[server_name][matrix] = {}
        # here be dragons
        xform = "%s_st-%s" % (row[2], row[3])
        results[server_name][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]] = {}
        results[server_name][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['cg_execution_time_mean'] = row[4]
        results[server_name][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['cg_execution_time_stddev'] = row[5]
        results[server_name][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['l2_refs_mean'] = row[6]
        results[server_name][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['l2_refs_stddev'] = row[7]
        results[server_name][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['l2_miss_mean'] = row[8]
        results[server_name][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['l2_miss_stddev'] = row[9]
        results[server_name][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['l3_refs_mean'] = row[10]
        results[server_name][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['l3_refs_stddev'] = row[11]
        results[server_name][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['l3_miss_mean'] = row[12]
        results[server_name][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['l3_miss_stddev'] = row[13]

# Average performance for groups of matrices
def create_group_perf_plot(graph_name, confs = None):
    if confs is None:
        confs = CONF_SPEEDUP_MIN_SORT_LUT
    # Groups of matrices that will have their average performance shown in plot
    groups = [
        ('all', results['vilje'].keys()),
        ('low_nnz_group', ['LF10000', 'nasa2146', 'nasa2910', 'nasa4704', 'sts4098']),
        ('middle_group', ['2cubes_sphere', 'offshore', 'parabolic_fem', 'thermal1', 'thermal2']),
        ('belt_group', ['af_5_k101', 'af_shell3', 'bone010', 'boneS01', 'boneS10', 'gyro', 'nasasrb', 'olafu', 'Pres_Poisson', 'raefsky4', 'smt']),
        ('csx_optimized_group', ['af_5_k101', 'af_shell3', 'bone010', 'boneS01', 'boneS10']),
        ('semi_optimized_group', ['gyro', 'LF10000', 'nasasrb', 'offshore', 'parabolic_fem', 'smt', 'sts4098']),
        ('csr_optimized_group', ['2cubes_sphere', 'nasa2146', 'nasa2910', 'nasa4704', 'olafu', 'Pres_Poisson', 'raefsky4', 'thermal1', 'thermal2'])
    ]
    data = []
    index = 0
    for (group_name,matrices) in groups:
        for server in ['card', 'vilje']:
            newgroup = True
            for conf in confs:
                runtime = 0
                for matrix in matrices:
                    runtime = runtime + results[server][matrix][conf]['cg_execution_time_mean']
                runtime = runtime / len(matrices)
                if newgroup:
                    data.append(("%s_%s" % (server, group_name), []))
                    newgroup = False
                data[index][1].append(runtime)
            index = index + 1
    plt.figure()
    ind = np.arange(len(confs))
    i = 0
    print graph_name
    for (group_name,values) in data:
        if i % 2 == 0:
            plt.plot(ind, values, label=group_name, c=bar_colors[i/2])
            (pcc,ttpv) = pearsonr(data[i][1], data[i+1][1])
            print "%s, %s: %f" % (data[i][0], data[i+1][0], pcc)
        else:
            plt.plot(ind, values, ls='--', label=group_name, c=bar_colors[i/2])
        i += 1
    plt.ylabel('Average runtime (s)')
    plt.xticks(ind, confs, rotation=30, fontsize='10')
    plt.grid(which='both')
    plt.legend(loc='best', prop={'size':10})
    plt.savefig("%s/%s.png" % (OUTPUT_DIR, graph_name))
    plt.close()

# Average miss rate for groups of matrices
def create_group_miss_plot(graph_name, confs = None):
    if confs is None:
        confs = CONF_SPEEDUP_MIN_SORT_LUT
    # Groups of matrices that will have their average miss rate shown in plot
    groups = [
        ('all', results['vilje'].keys()),
        ('low_nnz_group', ['LF10000', 'nasa2146', 'nasa2910', 'nasa4704', 'sts4098']),
        ('middle_group', ['2cubes_sphere', 'offshore', 'parabolic_fem', 'thermal1', 'thermal2']),
        ('belt_group', ['af_5_k101', 'af_shell3', 'bone010', 'boneS01', 'boneS10', 'gyro', 'nasasrb', 'olafu', 'Pres_Poisson', 'raefsky4', 'smt']),
        ('csx_optimized_group', ['af_5_k101', 'af_shell3', 'bone010', 'boneS01', 'boneS10']),
        ('semi_optimized_group', ['gyro', 'LF10000', 'nasasrb', 'offshore', 'parabolic_fem', 'smt', 'sts4098']),
        ('csr_optimized_group', ['2cubes_sphere', 'nasa2146', 'nasa2910', 'nasa4704', 'olafu', 'Pres_Poisson', 'raefsky4', 'thermal1', 'thermal2'])
    ]
    data = []
    index = 0
    for (group_name,matrices) in groups:
        for server in ['card', 'vilje']:
            newgroup = True
            for conf in confs:
                miss_rate = 0
                for matrix in matrices:
                    miss_rate = miss_rate + results[server][matrix][conf]['l3_miss_mean']/results[server][matrix][conf]['l3_refs_mean']
                miss_rate = (miss_rate / len(matrices))*100
                if newgroup:
                    data.append(("%s_%s" % (server, group_name), []))
                    newgroup = False
                data[index][1].append(miss_rate)
            index = index + 1
    fig = plt.figure()
    ax = fig.add_subplot(111)
    ind = np.arange(len(confs))
    i = 0
    for (group_name,values) in data:
        if i % 2 == 0:
            ax.plot(ind, values, label=group_name, c=bar_colors[i/2])
        else:
            ax.plot(ind, values, ls='--', label=group_name, c=bar_colors[i/2])
        i += 1
    plt.ylabel('L3 miss rate (%)')
    plt.xticks(ind, confs, rotation=30, fontsize='10')
    plt.grid(which='both')
    box = ax.get_position()
    ax.set_position([box.x0, box.y0, box.width * 0.65, box.height])
    ax.legend(loc=7, prop={'size':10}, bbox_to_anchor=(1.7,0.5))
    plt.savefig("%s/%s.png" % (OUTPUT_DIR, graph_name))
    plt.close()

for i in range(1,len(sys.argv),2):
    if (sys.argv[i] == "-card"):
        CARD_INPUT = sys.argv[i+1]
    elif (sys.argv[i] == "-vilje"):
        VILJE_INPUT = sys.argv[i+1]
    elif (sys.argv[i] == "-out"):
        OUTPUT_DIR = sys.argv[i+1]
    else:
        print "Invalid argument: %s, usage: %s -card carddb -vilje viljedb -out output_dir" % (sys.argv[i], sys.argv[0])
        sys.exit(1)

if not (CARD_INPUT and VILJE_INPUT):
    print "No input specified! usage: %s -card carddb -vilje viljedb -out output_dir" % sys.argv[0]
    sys.exit(1)

if not OUTPUT_DIR:
    print "No output directory specified! usage: %s -card carddb -vilje viljedb -out graph_dir" % sys.argv[0]
    sys.exit(1)

if CARD_INPUT and not (os.path.exists(CARD_INPUT) and os.path.isfile(CARD_INPUT)):
    print "Card database does not exist or is not a file: %s" % CARD_INPUT
    sys.exit(1)

if VILJE_INPUT and not (os.path.exists(VILJE_INPUT) and os.path.isfile(VILJE_INPUT)):
    print "Vilje database does not exist or is not a file: %s" % VILJE_INPUT
    sys.exit(1)

if not os.path.exists(OUTPUT_DIR):
    os.makedirs(OUTPUT_DIR)

if (os.path.exists(OUTPUT_DIR) and not os.path.isdir(OUTPUT_DIR)):
    print "Output directory is not a directory: %s" % OUTPUT_DIR
    sys.exit(1)

query = """
    SELECT
        matrix, mt_conf, xform_conf, enable_stat_preprocessing,
        cg_execution_time_mean, cg_execution_time_stddev,
        papi_l2_refs_mean, papi_l2_refs_stddev, papi_l2_miss_mean, papi_l2_miss_stddev,
        papi_l3_refs_mean, papi_l3_refs_stddev, papi_l3_miss_mean, papi_l3_miss_stddev
    FROM
        results
    ORDER BY
        matrix;"""

(conn,c) = connect_to_db(CARD_INPUT)
fill_dbresults(c.execute(query), 'card')
conn.close()

(conn,c) = connect_to_db(VILJE_INPUT)
fill_dbresults(c.execute(query), 'vilje')
conn.close()

create_group_perf_plot('csr_cvv_runtime', CONF_CSR_MIN_SORT_LUT)
create_group_perf_plot('csx_cvv_runtime', CONF_CSX_MIN_SORT_LUT)
#create_group_miss_plot('csr_cvv_miss', CONF_CSR_MIN_SORT_LUT)
#create_group_miss_plot('csx_cvv_miss', CONF_CSX_MIN_SORT_LUT)


sys.exit()
