import matplotlib.pyplot as plt
import sys
import os
import numpy as np
import sqlite3
import re
import glob
from math import sqrt
import datetime

from plot_configurations import *

results = {}
DB_INPUT = None
OUTPUT_DIR = None
HALLGEIR_CONST = 0.001

bar_colors = [ 'b', 'r', 'g', 'c', 'm', 'y', 'k']

# Return conn and cursor
def connect_to_db(dbname):
    conn = sqlite3.connect(dbname)
    return (conn,conn.cursor())

# Create dictionary structure
def fill_dbresults(db_data):
    # row[0] = matrix, row[1] = mt_conf, row[2] = xform_conf, row[3] = enable_stat_preprocessing, row[4] = cpufreq
    # row[5] = cg_execution_time_mean, row[6] = cg_execution_time_stddev, row[7] = energy_pkg_mean, row[8] = energy_pkg_stddev,
    # row[9] = papi_l2_refs_mean, row[10] = papi_l2_refs_stddev, row[11] = papi_l2_miss_mean, row[12] = papi_l2_miss_stddev,
    # row[13] = papi_l3_refs_mean, row[14] = papi_l3_refs_stddev, row[15] = papi_l3_miss_mean, row[16] = papi_l3_miss_stddev,
    for row in db_data:
        # remove path
        matrix = re.sub('\.\/csx\/matrices\/\S+\/', '', row[0])
        # remove .mtx.sorted
        matrix = matrix[:-11]
        cpufreq = row[4]
        if cpufreq not in results.keys():
            results[cpufreq] = {}
        if matrix not in results[cpufreq].keys():
            results[cpufreq][matrix] = {}
        # here be dragons
        xform = "%s_st-%s" % (row[2], row[3])
        results[cpufreq][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]] = {}
        results[cpufreq][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['cg_execution_time_mean'] = row[5]
        results[cpufreq][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['cg_execution_time_stddev'] = row[6]
        results[cpufreq][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['energy_pkg_mean'] = row[7] * HALLGEIR_CONST
        results[cpufreq][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['energy_pkg_stddev'] = row[8] * HALLGEIR_CONST
        results[cpufreq][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['l2_refs_mean'] = row[9]
        results[cpufreq][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['l2_refs_stddev'] = row[10]
        results[cpufreq][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['l2_miss_mean'] = row[11]
        results[cpufreq][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['l2_miss_stddev'] = row[12]
        results[cpufreq][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['l3_refs_mean'] = row[13]
        results[cpufreq][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['l3_refs_stddev'] = row[14]
        results[cpufreq][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['l3_miss_mean'] = row[15]
        results[cpufreq][matrix][MTCONF_LUT[row[1]]+XFORM_LUT[xform]]['l3_miss_stddev'] = row[16]

# Average performance for groups of matrices
def create_group_perf_plot(graph_name, confs = None):
    if confs is None:
        confs = CONF_SPEEDUP_MIN_SORT_LUT
    # Groups of matrices that will have their average performance shown in plot
    groups = [
        ('all', results[2600].keys()),
#        ('low_nnz_group', ['LF10000', 'nasa2146', 'nasa2910', 'nasa4704', 'sts4098']),
#        ('middle_group', ['2cubes_sphere', 'offshore', 'parabolic_fem', 'thermal1', 'thermal2']),
#        ('belt_group', ['af_5_k101', 'af_shell3', 'bone010', 'boneS01', 'boneS10', 'gyro', 'nasasrb', 'olafu', 'Pres_Poisson', 'raefsky4', 'smt']),
#        ('csx_optimized_group', ['af_5_k101', 'af_shell3', 'bone010', 'boneS01', 'boneS10']),
#        ('semi_optimized_group', ['gyro', 'LF10000', 'nasasrb', 'offshore', 'parabolic_fem', 'smt', 'sts4098']),
#        ('csr_optimized_group', ['2cubes_sphere', 'nasa2146', 'nasa2910', 'nasa4704', 'olafu', 'Pres_Poisson', 'raefsky4', 'thermal1', 'thermal2'])
    ]
    data = []
    index = 0
    for (group_name,matrices) in groups:
        for cpufreq in sorted(results.keys()):
            newgroup = True
            for conf in confs:
                runtime = 0
                for matrix in matrices:
                    runtime = runtime + results[cpufreq][matrix][conf]['cg_execution_time_mean']
                runtime = runtime / len(matrices)
                if newgroup:
                    data.append(("%s_%s" % (cpufreq, group_name), []))
                    newgroup = False
                data[index][1].append(runtime)
            index = index + 1
    plt.figure()
    ind = np.arange(len(confs))
    i = 0
    for (group_name,values) in data:
        if (i < 7):
            plt.plot(ind, values, label=group_name)
        else:
            plt.plot(ind, values, label=group_name, ls='--')
        i += 1
    plt.ylabel('Average runtime (s)')
    plt.xticks(ind, confs, rotation=30, fontsize='10')
    plt.grid(which='both')
    plt.legend(loc='best', prop={'size':10})
    plt.savefig("%s/%s.png" % (OUTPUT_DIR, graph_name))
    plt.close()

# Average performance for groups of matrices
def create_group_energy_plot(graph_name, confs = None):
    if confs is None:
        confs = CONF_SPEEDUP_MIN_SORT_LUT
    # Groups of matrices that will have their average performance shown in plot
    groups = [
        ('all', results[2600].keys()),
#        ('low_nnz_group', ['LF10000', 'nasa2146', 'nasa2910', 'nasa4704', 'sts4098']),
#        ('middle_group', ['2cubes_sphere', 'offshore', 'parabolic_fem', 'thermal1', 'thermal2']),
#        ('belt_group', ['af_5_k101', 'af_shell3', 'bone010', 'boneS01', 'boneS10', 'gyro', 'nasasrb', 'olafu', 'Pres_Poisson', 'raefsky4', 'smt']),
#        ('csx_optimized_group', ['af_5_k101', 'af_shell3', 'bone010', 'boneS01', 'boneS10']),
#        ('semi_optimized_group', ['gyro', 'LF10000', 'nasasrb', 'offshore', 'parabolic_fem', 'smt', 'sts4098']),
#        ('csr_optimized_group', ['2cubes_sphere', 'nasa2146', 'nasa2910', 'nasa4704', 'olafu', 'Pres_Poisson', 'raefsky4', 'thermal1', 'thermal2'])
    ]
    data = []
    index = 0
    for (group_name,matrices) in groups:
        for cpufreq in sorted(results.keys()):
            newgroup = True
            for conf in confs:
                runtime = 0
                for matrix in matrices:
                    runtime = runtime + results[cpufreq][matrix][conf]['energy_pkg_mean']
                runtime = runtime / len(matrices)
                if newgroup:
                    data.append(("%s_%s" % (cpufreq, group_name), []))
                    newgroup = False
                data[index][1].append(runtime)
            index = index + 1
    plt.figure()
    ind = np.arange(len(confs))
    i = 0
    for (group_name,values) in data:
        if (i < 7):
            plt.plot(ind, values, label=group_name)
        else:
            plt.plot(ind, values, label=group_name, ls='--')
        i += 1
    plt.ylabel('Average energy consumed (J)')
    plt.ylim(180,900)
    plt.grid(which='both')
    plt.xticks(ind, confs, rotation=30, fontsize='10')
    plt.legend(loc='best', prop={'size':10})
    plt.savefig("%s/%s.png" % (OUTPUT_DIR, graph_name))
    plt.close()

# Average performance for groups of matrices
def create_group_edp_plot(graph_name, confs = None):
    if confs is None:
        confs = CONF_SPEEDUP_MIN_SORT_LUT
    # Groups of matrices that will have their average performance shown in plot
    groups = [
        ('all', results[2600].keys()),
#        ('low_nnz_group', ['LF10000', 'nasa2146', 'nasa2910', 'nasa4704', 'sts4098']),
#        ('middle_group', ['2cubes_sphere', 'offshore', 'parabolic_fem', 'thermal1', 'thermal2']),
#        ('belt_group', ['af_5_k101', 'af_shell3', 'bone010', 'boneS01', 'boneS10', 'gyro', 'nasasrb', 'olafu', 'Pres_Poisson', 'raefsky4', 'smt']),
#        ('csx_optimized_group', ['af_5_k101', 'af_shell3', 'bone010', 'boneS01', 'boneS10']),
#        ('semi_optimized_group', ['gyro', 'LF10000', 'nasasrb', 'offshore', 'parabolic_fem', 'smt', 'sts4098']),
#        ('csr_optimized_group', ['2cubes_sphere', 'nasa2146', 'nasa2910', 'nasa4704', 'olafu', 'Pres_Poisson', 'raefsky4', 'thermal1', 'thermal2'])
    ]
    data = []
    index = 0
    for (group_name,matrices) in groups:
        for cpufreq in sorted(results.keys()):
            newgroup = True
            for conf in confs:
                runtime = 0
                for matrix in matrices:
                    runtime = runtime + results[cpufreq][matrix][conf]['energy_pkg_mean']*pow(results[cpufreq][matrix][conf]['cg_execution_time_mean'], 2)
                runtime = runtime / len(matrices)
                runtime *= 100
                if newgroup:
                    data.append(("%s_%s" % (cpufreq, group_name), []))
                    newgroup = False
                data[index][1].append(runtime)
            index = index + 1
    plt.figure()
    ind = np.arange(len(confs))
    i = 0
    print data
    for (group_name,values) in data:
        if (i < 7):
            plt.plot(ind, values, label=group_name)
        else:
            plt.plot(ind, values, label=group_name, ls='--')
        i += 1
    plt.ylabel('Energy Delay Product')
    plt.yscale('log')
    plt.grid(which='both')
    plt.xticks(ind, confs, rotation=30, fontsize='10')
    plt.legend(loc='best', prop={'size':10})
    plt.savefig("%s/%s.png" % (OUTPUT_DIR, graph_name))
    plt.close()

for i in range(1,len(sys.argv),2):
    if (sys.argv[i] == "-db"):
        DB_INPUT = sys.argv[i+1]
    elif (sys.argv[i] == "-out"):
        OUTPUT_DIR = sys.argv[i+1]
    else:
        print "Invalid argument: %s, usage: %s -db database -out output_dir" % (sys.argv[i], sys.argv[0])
        sys.exit(1)

if not DB_INPUT:
    print "No input specified! usage: %s -db database -out output_dir" % sys.argv[0]
    sys.exit(1)

if not OUTPUT_DIR:
    print "No output directory specified! usage: %s -db datebase -out graph_dir" % sys.argv[0]
    sys.exit(1)

if DB_INPUT and not (os.path.exists(DB_INPUT) and os.path.isfile(DB_INPUT)):
    print "Card database does not exist or is not a file: %s" % DB_INPUT
    sys.exit(1)

if not os.path.exists(OUTPUT_DIR):
    os.makedirs(OUTPUT_DIR)

if (os.path.exists(OUTPUT_DIR) and not os.path.isdir(OUTPUT_DIR)):
    print "Output directory is not a directory: %s" % OUTPUT_DIR
    sys.exit(1)

query = """
    SELECT
        matrix, mt_conf, xform_conf, enable_stat_preprocessing, cpufreq,
        cg_execution_time_mean, cg_execution_time_stddev, energy_pkg_mean, energy_pkg_stddev,
        papi_l2_refs_mean, papi_l2_refs_stddev, papi_l2_miss_mean, papi_l2_miss_stddev,
        papi_l3_refs_mean, papi_l3_refs_stddev, papi_l3_miss_mean, papi_l3_miss_stddev
    FROM
        results
    ORDER BY
        matrix;"""

(conn,c) = connect_to_db(DB_INPUT)
fill_dbresults(c.execute(query))
conn.close()

#create_group_perf_plot("csr_freq_runtime", CONF_CSR_MIN_SORT_LUT)
#create_group_perf_plot("csx_freq_runtime", CONF_CSX_MIN_SORT_LUT)
create_group_energy_plot("csr_freq_energy", CONF_CSR_MIN_SORT_LUT)
create_group_energy_plot("csx_freq_energy", CONF_CSX_MIN_SORT_LUT)
#create_group_edp_plot("csr_freq_edp", CONF_CSR_MIN_SORT_LUT)
#create_group_edp_plot("csx_freq_edp", CONF_CSX_MIN_SORT_LUT)

sys.exit()
