import matplotlib.pyplot as plt
import sys
import os
import numpy as np
import re
import glob
import scipy
from scipy.stats import pearsonr

from plot_configurations import *

YOKO_DIR = None
JCM_DIR = None
OUTPUT_DIR = None
HALLGEIR_CONST = 0.001
YOKO_CONST = 0.01
ITERATIONS = 1

# TODO: Redesign configs, also in test.py

def create_powertime_plots():
    plt.figure()
    powertime_confs = glob.glob("%s/*powertime_1.dat" % YOKO_DIR)

    for conf in powertime_confs:
        filename = re.sub("^(.+/)+", "", conf)
        m = re.match("(.+)_mt-([\d,]+)_xf-([\d,]+_st-\d)_powertime_1.dat", filename)
        matrix = m.group(1)
        mt_conf = m.group(2)
        xform_conf = m.group(3)
        f = open(conf, 'r')
        power_points = f.read().split('\t\n')
        power_points.pop()
        f.close()
        ind = np.arange(len(power_points))
        power_points[:] = [int(point) * YOKO_CONST for point in power_points]
        plt.plot(ind, power_points, label="Yokogawa", c='r')
        if JCM_DIR:
            #TODO: rewrite this for regular tests
            filename = "cg_exp_setup_%s" % (filename)
            filename = re.sub("_powertime_1.dat", "", filename)
            f = open("%s/%s" % (JCM_DIR, filename), 'r')
            power_points = f.read().split('\n')
            power_points.pop()
            power_points.pop(0)
            f.close()
            ind = np.arange(len(power_points))
            power_points[:] = [float(point) * HALLGEIR_CONST for point in power_points]
            plt.plot(ind, power_points, label="MSR", c='b')
#        for i in range (2,ITERATIONS+1):
#            filename = re.sub("powertime_1.dat", "powertime_%d.dat" % i, conf)
#            f = open(filename, 'r')
#            power_points = f.read().split('\t\n')
#            power_points.pop()
#            f.close()
#            ind = np.arange(len(power_points))
#            power_points[:] = [int(point) * YOKO_CONST for point in power_points]
#            plt.plot(ind, power_points, label="Run %d" % i)
        plt.xlabel('Timesteps (10e-1 s)')
        plt.ylabel('Power (W)')
        plt.title('Powertime for %s_%s%s' % (matrix, MTCONF_LUT[mt_conf], XFORM_LUT[xform_conf]))
        plt.grid('on')
        plt.legend(loc='best', prop={'size':10})
        plt.savefig('%s/%s_%s%s_powertime.png' % (OUTPUT_DIR, matrix, MTCONF_LUT[mt_conf], XFORM_LUT[xform_conf]))
        plt.clf()

def create_eng_from_powertime_plot(graph_name, confs = None):
    if not JCM_DIR:
        print "Need JCM_DIR to run this function"
        exit(1)

    if confs is None:
        confs = CONF_SORT_LUT

    results = {}
    powertime_confs = glob.glob("%s/*" % JCM_DIR)

    for conf in powertime_confs:
        filename = re.sub("^(.+/)+cg_exp_setup_", "", conf)
        m = re.match("(.+)_mt-([\d,]+)_xf-([\d,]+_st-\d)", filename)
        matrix = m.group(1)
        mt_conf = m.group(2)
        xform_conf = m.group(3)
        f = open(conf, 'r')
        power_points = f.read().split('\n')
        power_points.pop()
        power_points.pop(0)
        f.close()
        ind = np.arange(len(power_points))
        power_points[:] = [float(point) * HALLGEIR_CONST for point in power_points]

        energy = 0
        time = 0
        for point in power_points:
            energy += point*0.1
            time += 0.1
        if matrix not in results.keys():
            results[matrix] = {}
        results[matrix][MTCONF_LUT[mt_conf]+XFORM_LUT[xform_conf]+"_energy"] = energy
        results[matrix][MTCONF_LUT[mt_conf]+XFORM_LUT[xform_conf]+"_time"] = time

    groups = [
        ('all', MATRIX_LIST, ["R", "X"]),
        ('stat_prep', MATRIX_LIST_CSXS, ["R", "X", "Xs"]),
    ]
    data_time = []
    data_energy = []
    index = 0
    for (group_name,matrices, formats) in groups:
        for frm in formats:
            newgroup = True
            for conf in confs:
                time = 0
                energy = 0
                for matrix in matrices:
                    energy = energy + results[matrix][conf+frm+"_energy"]
                    time = time + results[matrix][conf+frm+"_time"]
                time = time / len(matrices)
                energy = energy / len(matrices)
                if newgroup:
                    data_time.append((group_name+"_CS"+frm, []))
                    data_energy.append((group_name+"_CS"+frm, []))
                    newgroup = False
                data_time[index][1].append(time)
                data_energy[index][1].append(energy)
            index = index + 1
    plt.figure()
    ind = np.arange(len(confs))
    for (group_name,values) in data_time:
        plt.plot(ind, values, label=group_name)
    plt.ylabel('Average runtime (s)')
#    plt.yscale('log')
    plt.xticks(ind, confs, rotation=30, fontsize='10')
    plt.grid(which='both')
    plt.legend(loc='best', prop={'size':10})
    plt.savefig("%s/%s_runtime.png" % (OUTPUT_DIR, graph_name))
    plt.close()
    plt.figure()
    ind = np.arange(len(confs))
    for (group_name,values) in data_energy:
        plt.plot(ind, values, label=group_name)
    plt.ylabel('Average energy consumption (J)')
#    plt.yscale('log')
    plt.xticks(ind, confs, rotation=30, fontsize='10')
    plt.grid(which='both')
    plt.legend(loc='best', prop={'size':10})
    plt.savefig("%s/%s_energy.png" % (OUTPUT_DIR, graph_name))
    plt.close()

def pearson_corr_coeff():
    if not JCM_DIR:
        print "Needs both Yoko and JCM dir"
        exit(1)
    powertime_confs = glob.glob("%s/*powertime_1.dat" % YOKO_DIR)

    results = {}

    for conf in powertime_confs:
        filename = re.sub("^(.+/)+", "", conf)
        m = re.match("(.+)_mt-([\d,]+)_xf-([\d,]+_st-\d)_powertime_1.dat", filename)
        matrix = m.group(1)
        mt_conf = m.group(2)
        xform_conf = m.group(3)
        f = open(conf, 'r')
        yoko_points = f.read().split('\t\n')
        yoko_points.pop()
        f.close()
        yoko_points[:] = [int(point) * YOKO_CONST for point in yoko_points]
        yoko_len = len(yoko_points)
        if yoko_len < 10:
            continue
        filename = "cg_exp_setup_%s" % (filename)
        filename = re.sub("_powertime_1.dat", "", filename)
        f = open("%s/%s" % (JCM_DIR, filename), 'r')
        jcm_points = f.read().split('\n')
        jcm_points.pop()
        jcm_points.pop(0)
        f.close()
        jcm_points[:] = [float(point) * HALLGEIR_CONST for point in jcm_points]
        jcm_len = len(jcm_points)
        if jcm_len < 10:
            continue
        if matrix == "nasa2146":
            print "YOLO! %d %d, %s" % (jcm_len, yoko_len,MTCONF_LUT[mt_conf]+XFORM_LUT[xform_conf])
        if jcm_len > yoko_len:
            for i in range(0,(jcm_len-yoko_len)):
                jcm_points.pop()
        elif yoko_len > jcm_len:
            for i in range(0,(yoko_len-jcm_len)):
                yoko_points.pop()
        (pcc,ttpv) = pearsonr(yoko_points, jcm_points)
        if matrix not in results.keys():
            results[matrix] = []
        results[matrix].append(pcc)

    print "\\begin{table}[h!]"
    print "\\center"
    print "\\begin{tabular}{|l|r|r|}"
    print "\\hline"
    print "\\textbf{Name} & \\textbf{average pcc} & \\textbf{number of runs} \\\\\\hline"
    for matrix in MATRIX_LIST:
        if matrix not in results.keys():
            continue
        avg_pcc = 0
        for pcc in results[matrix]:
            avg_pcc += pcc
        avg_pcc /= len(results[matrix])
        print "%s & %f & %d \\\\\\hline" % (matrix.replace("_", "\\_"), avg_pcc, len(results[matrix]))
    print "\\end{tabular}"
    print "\\end{table}"

for i in range(1,len(sys.argv),2):
    if (sys.argv[i] == "-yd"):
        YOKO_DIR = sys.argv[i+1]
    elif (sys.argv[i] == "-jd"):
        JCM_DIR = sys.argv[i+1]
    elif (sys.argv[i] == "-out"):
        OUTPUT_DIR = sys.argv[i+1]
    else:
        print "Invalid argument: %s, usage: %s -yd yoko_dir [-jd jcm_dir] -out graph_dir" % (sys.argv[i], sys.argv[0])
        sys.exit(1)

if not YOKO_DIR:
    print "No yoko input directory specified! usage: %s -yd yoko_dir [-jd jcm_dir] -out graph_dir" % sys.argv[0]
    sys.exit(1)

if not OUTPUT_DIR:
    print "No output directory specified! usage: %s -yd yoko_dir [-jd jcm_dir] -out graph_dir" % sys.argv[0]
    sys.exit(1)

if not (os.path.exists(YOKO_DIR) and os.path.isdir(YOKO_DIR)):
    print "Input directory does not exist or is not a directory: %s" % YOKO_DIR
    sys.exit(1)

if JCM_DIR and not (os.path.exists(JCM_DIR) and os.path.isdir(JCM_DIR)):
    print "Input directory does not exist or is not a directory: %s" % JCM_DIR
    sys.exit(1)

if not os.path.exists(OUTPUT_DIR):
    os.makedirs(OUTPUT_DIR)

if (os.path.exists(OUTPUT_DIR) and not os.path.isdir(OUTPUT_DIR)):
    print "Output directory is not a directory: %s" % OUTPUT_DIR
    sys.exit(1)

#create_powertime_plots()
#create_eng_from_powertime_plot("full_app", CONF_SPEEDUP_MIN_SORT_LUT)
pearson_corr_coeff()

sys.exit()
